@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: dvbsdffdevice.c 2.30 2011/08/27 11:33:57 kls Exp $
+ * $Id: dvbsdffdevice.c 2.33 2012/03/11 13:32:42 kls Exp $
  */
 
 #include "dvbsdffdevice.h"
@@ -66,10 +66,6 @@ cDvbSdFfDevice::cDvbSdFfDevice(int Adapter, int Frontend, bool OutputOnly)
         fclose(f);
      }
   devVideoIndex = devVideoOffset >= 0 ? devVideoOffset++ : -1;
-
-  // Video format:
-
-  SetVideoFormat(Setup.VideoFormat);
 }
 
 cDvbSdFfDevice::~cDvbSdFfDevice()
@@ -425,13 +421,6 @@ bool cDvbSdFfDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
   if (!cDvbDevice::SetChannelDevice(Channel, LiveView))
      return false;
 
-  // If this channel switch was requested by the EITScanner we don't wait for
-  // a lock and don't set any live PIDs (the EITScanner will wait for the lock
-  // by itself before setting any filters):
-
-  if (EITScanner.UsesDevice(this)) //XXX
-     return true;
-
   // PID settings:
 
   if (TurnOnLivePIDs) {
@@ -442,7 +431,7 @@ bool cDvbSdFfDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
         }
      if (IsPrimaryDevice())
         AddPid(Channel->Tpid(), ptTeletext);
-     CHECK(ioctl(fd_audio, AUDIO_SET_MUTE, true)); // actually one would expect 'false' here, but according to Marco Schller <marco@lordzodiac.de> this works
+     CHECK(ioctl(fd_audio, AUDIO_SET_MUTE, true)); // actually one would expect 'false' here, but according to Marco Schluessler <marco@lordzodiac.de> this works
                                                    // to avoid missing audio after replaying a DVD; with 'false' there is an audio disturbance when switching
                                                    // between two channels on the same transponder on DVB-S
      CHECK(ioctl(fd_audio, AUDIO_SET_AV_SYNC, true));
