@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 2.19 2011/08/26 12:44:21 kls Exp $
+ * $Id: channels.c 2.22 2012/04/01 09:27:08 kls Exp $
  */
 
 #include "channels.h"
@@ -619,7 +619,8 @@ bool cChannel::Parse(const char *s)
                        }
                     else
                        *alangs[NumApids] = 0;
-                    apids[NumApids++] = strtol(q, NULL, 10);
+                    if ((apids[NumApids] = strtol(q, NULL, 10)) != 0)
+                       NumApids++;
                     }
                  else
                     esyslog("ERROR: too many APIDs!"); // no need to set ok to 'false'
@@ -647,7 +648,8 @@ bool cChannel::Parse(const char *s)
                           }
                        else
                           *dlangs[NumDpids] = 0;
-                       dpids[NumDpids++] = strtol(q, NULL, 10);
+                       if ((dpids[NumDpids] = strtol(q, NULL, 10)) != 0)
+                          NumDpids++;
                        }
                     else
                        esyslog("ERROR: too many DPIDs!"); // no need to set ok to 'false'
@@ -758,6 +760,8 @@ cChannels Channels;
 cChannels::cChannels(void)
 {
   maxNumber = 0;
+  maxChannelNameLength = 0;
+  maxShortChannelNameLength = 0;
   modified = CHANNELSMOD_NONE;
 }
 
@@ -935,9 +939,32 @@ bool cChannels::SwitchTo(int Number)
   return channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true);
 }
 
+int cChannels::MaxChannelNameLength(void)
+{
+  if (!maxChannelNameLength) {
+     for (cChannel *channel = First(); channel; channel = Next(channel)) {
+         if (!channel->GroupSep())
+            maxChannelNameLength = max(Utf8StrLen(channel->Name()), maxChannelNameLength);
+         }
+     }
+  return maxChannelNameLength;
+}
+
+int cChannels::MaxShortChannelNameLength(void)
+{
+  if (!maxShortChannelNameLength) {
+     for (cChannel *channel = First(); channel; channel = Next(channel)) {
+         if (!channel->GroupSep())
+            maxShortChannelNameLength = max(Utf8StrLen(channel->ShortName(true)), maxShortChannelNameLength);
+         }
+     }
+  return maxShortChannelNameLength;
+}
+
 void cChannels::SetModified(bool ByUser)
 {
   modified = ByUser ? CHANNELSMOD_USER : !modified ? CHANNELSMOD_AUTO : modified;
+  maxChannelNameLength = maxShortChannelNameLength = 0;
 }
 
 int cChannels::Modified(void)
