@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 2.39 2012/01/14 13:03:53 kls Exp $
+ * $Id: config.h 2.47 2012/04/15 10:45:32 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.7.23"
-#define VDRVERSNUM   10723  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.7.28"
+#define VDRVERSNUM   10728  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.7.23"
-#define APIVERSNUM   10723  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "1.7.28"
+#define APIVERSNUM   10728  // Version * 10000 + Major * 100 + Minor
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -36,8 +36,12 @@
 // plugins to work with newer versions of the core VDR as long as no
 // VDR header files have changed.
 
-#define MAXPRIORITY 99
-#define MAXLIFETIME 99
+#define MAXPRIORITY       99
+#define MINPRIORITY       (-MAXPRIORITY)
+#define LIVEPRIORITY      0                  // priority used when selecting a device for live viewing
+#define TRANSFERPRIORITY  (LIVEPRIORITY - 1) // priority used for actual local Transfer Mode
+#define IDLEPRIORITY      (MINPRIORITY - 1)  // priority of an idle device
+#define MAXLIFETIME       99
 
 #define MINOSDWIDTH   480
 #define MAXOSDWIDTH  1920
@@ -48,6 +52,16 @@
 #define MaxSkinName 16
 #define MaxThemeName 16
 
+// Basically VDR works according to the DVB standard, but there are countries/providers
+// that use other standards, which in some details deviate from the DVB standard.
+// This makes it necessary to handle things differently in some areas, depending on
+// which standard is actually used. The following macros are used to distinguish
+// these cases (make sure to adjust cMenuSetupDVB::standardComplianceTexts accordingly
+// when adding a new standard):
+
+#define STANDARD_DVB       0
+#define STANDARD_ANSISCTE  1
+
 typedef uint32_t in_addr_t; //XXX from /usr/include/netinet/in.h (apparently this is not defined on systems with glibc < 2.2)
 
 class cSVDRPhost : public cListObject {
@@ -251,6 +265,7 @@ public:
   int SetSystemTime;
   int TimeSource;
   int TimeTransponder;
+  int StandardCompliance;
   int MarginStart, MarginStop;
   int AudioLanguages[I18N_MAX_LANGUAGES + 1];
   int DisplaySubtitles;
@@ -264,7 +279,6 @@ public:
   int SVDRPTimeout;
   int ZapTimeout;
   int ChannelEntryTimeout;
-  int PrimaryLimit;
   int DefaultPriority, DefaultLifetime;
   int PausePriority, PauseLifetime;
   int PauseKeyHandling;
