@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.c 2.56 2012/01/15 14:31:47 kls Exp $
+ * $Id: dvbdevice.c 2.71 2012/05/09 08:33:59 kls Exp $
  */
 
 #include "dvbdevice.h"
@@ -285,6 +285,7 @@ class cDvbTuner : public cThread {
 private:
   static cMutex bondMutex;
   enum eTunerStatus { tsIdle, tsSet, tsTuned, tsLocked };
+  int frontendType;
   const cDvbDevice *device;
   int fd_frontend;
   int adapter, frontend;
@@ -302,7 +303,6 @@ private:
   cCondVar newSet;
   cDvbTuner *bondedTuner;
   bool bondedMaster;
-  bool bondedMasterFailed;
   bool SetFrontendType(const cChannel *Channel);
   cString GetBondingParams(const cChannel *Channel = NULL) const;
   void ClearEventQueue(void) const;
@@ -314,6 +314,7 @@ private:
 public:
   cDvbTuner(const cDvbDevice *Device, int Fd_Frontend, int Adapter, int Frontend);
   virtual ~cDvbTuner();
+  int FrontendType(void) const { return frontendType; }
   bool Bond(cDvbTuner *Tuner);
   void UnBond(void);
   bool BondingOk(const cChannel *Channel, bool ConsiderOccupied = false) const;
@@ -331,6 +332,7 @@ cMutex cDvbTuner::bondMutex;
 
 cDvbTuner::cDvbTuner(const cDvbDevice *Device, int Fd_Frontend, int Adapter, int Frontend)
 {
+  frontendType = SYS_UNDEFINED;
   device = Device;
   fd_frontend = Fd_Frontend;
   adapter = Adapter;
@@ -345,7 +347,6 @@ cDvbTuner::cDvbTuner(const cDvbDevice *Device, int Fd_Frontend, int Adapter, int
   tunerStatus = tsIdle;
   bondedTuner = NULL;
   bondedMaster = false;
-  bondedMasterFailed = false;
   SetDescription("tuner on frontend %d/%d", adapter, frontend);
   Start();
 }
@@ -420,7 +421,7 @@ bool cDvbTuner::BondingOk(const cChannel *Channel, bool ConsiderOccupied) const
   if (cDvbTuner *t = bondedTuner) {
      cString BondingParams = GetBondingParams(Channel);
      do {
-        if (t->device->Receiving() || t->tunerStatus != tsIdle && (t->device == cDevice::ActualDevice() || ConsiderOccupied && t->device->Occupied())) {
+        if (t->device->Priority() > IDLEPRIORITY || ConsiderOccupied && t->device->Occupied()) {
            if (strcmp(BondingParams, t->GetBondingParams()) != 0)
               return false;
            }
@@ -435,12 +436,8 @@ cDvbTuner *cDvbTuner::GetBondedMaster(void)
   if (!bondedTuner)
      return this; // an unbonded tuner is always "master"
   cMutexLock MutexLock(&bondMutex);
-  if (bondedMaster) {
-     if (!bondedMasterFailed)
-        return this;
-     else
-        bondedMaster = false;
-     }
+  if (bondedMaster)
+     return this;
   // This tuner is bonded, but it's not the master, so let's see if there is a master at all:
   if (cDvbTuner *t = bondedTuner) {
      while (t != this) {
@@ -450,18 +447,9 @@ cDvbTuner *cDvbTuner::GetBondedMaster(void)
            }
      }
   // None of the other bonded tuners is master, so make this one the master:
-  cDvbTuner *t = this;
-  if (bondedMasterFailed) {
-     // This one has failed, so switch to the next one:
-     t = bondedTuner;
-     t->bondedMasterFailed = false;
-     cMutexLock MutexLock(&t->mutex);
-     t->channel = channel;
-     t->tunerStatus = tsSet;
-     }
-  t->bondedMaster = true;
-  dsyslog("tuner %d/%d is now bonded master", t->adapter, t->frontend);
-  return t;
+  bondedMaster = true;
+  dsyslog("tuner %d/%d is now bonded master", adapter, frontend);
+  return this;
 }
 
 bool cDvbTuner::IsTunedTo(const cChannel *Channel) const
@@ -502,6 +490,8 @@ void cDvbTuner::SetChannel(const cChannel *Channel)
      tunerStatus = tsIdle;
      ResetToneAndVoltage();
      }
+  if (bondedTuner && device->IsPrimaryDevice())
+     cDevice::PrimaryDevice()->DelLivePids(); // 'device' is const, so we must do it this way
 }
 
 bool cDvbTuner::Locked(int TimeoutMs)
@@ -731,7 +721,7 @@ bool cDvbTuner::SetFrontend(void)
   cDvbTransponderParameters dtp(channel.Parameters());
 
   // Determine the required frontend type:
-  int frontendType = GetRequiredDeliverySystem(&channel, &dtp);
+  frontendType = GetRequiredDeliverySystem(&channel, &dtp);
   if (frontendType == SYS_UNDEFINED)
      return false;
 
@@ -857,6 +847,7 @@ void cDvbTuner::Action(void)
         if (GetFrontendStatus(NewStatus))
            Status = NewStatus;
         cMutexLock MutexLock(&mutex);
+        int WaitTime = 1000;
         switch (tunerStatus) {
           case tsIdle:
                break;
@@ -872,11 +863,9 @@ void cDvbTuner::Action(void)
                      isyslog("frontend %d/%d timed out while tuning to channel %d, tp %d", adapter, frontend, channel.Number(), channel.Transponder());
                      lastTimeoutReport = time(NULL);
                      }
-                  cMutexLock MutexLock(&bondMutex);
-                  if (bondedTuner && bondedMaster)
-                     bondedMasterFailed = true; // give an other tuner a chance in case the sat cable was disconnected
                   continue;
                   }
+               WaitTime = 100; // allows for a quick change from tsTuned to tsLocked
           case tsLocked:
                if (Status & FE_REINIT) {
                   tunerStatus = tsSet;
@@ -905,9 +894,7 @@ void cDvbTuner::Action(void)
                break;
           default: esyslog("ERROR: unknown tuner status %d", tunerStatus);
           }
-
-        if (tunerStatus != tsTuned)
-           newSet.TimedWait(mutex, 1000);
+        newSet.TimedWait(mutex, WaitTime);
         }
 }
 
@@ -975,7 +962,7 @@ int cDvbDevice::setTransferModeForDolbyDigital = 1;
 cMutex cDvbDevice::bondMutex;
 
 const char *DeliverySystemNames[] = {
-  "UNDEFINED",
+  "",
   "DVB-C",
   "DVB-C",
   "DVB-T",
@@ -1006,6 +993,7 @@ cDvbDevice::cDvbDevice(int Adapter, int Frontend)
   numModulations = 0;
   bondedDevice = NULL;
   needsDetachBondedReceivers = false;
+  tsBuffer = NULL;
 
   // Devices that are present on all card types:
 
@@ -1045,7 +1033,7 @@ cDvbDevice::~cDvbDevice()
 
 cString cDvbDevice::DvbName(const char *Name, int Adapter, int Frontend)
 {
-  return cString::sprintf("%s%d/%s%d", DEV_DVB_ADAPTER, Adapter, Name, Frontend);
+  return cString::sprintf("%s/%s%d/%s%d", DEV_DVB_BASE, DEV_DVB_ADAPTER, Adapter, Name, Frontend);
 }
 
 int cDvbDevice::DvbOpen(const char *Name, int Adapter, int Frontend, int Mode, bool ReportError)
@@ -1087,34 +1075,69 @@ bool cDvbDevice::Probe(int Adapter, int Frontend)
   return true;
 }
 
+cString cDvbDevice::DeviceType(void) const
+{
+  if (dvbTuner) {
+     if (dvbTuner->FrontendType() != SYS_UNDEFINED)
+        return DeliverySystemNames[dvbTuner->FrontendType()];
+     if (numDeliverySystems)
+        return DeliverySystemNames[deliverySystems[0]]; // to have some reasonable default
+     }
+  return "";
+}
+
+cString cDvbDevice::DeviceName(void) const
+{
+  return frontendInfo.name;
+}
+
 bool cDvbDevice::Initialize(void)
 {
   new cDvbSourceParam('A', "ATSC");
   new cDvbSourceParam('C', "DVB-C");
   new cDvbSourceParam('S', "DVB-S");
   new cDvbSourceParam('T', "DVB-T");
+  cStringList Nodes;
+  cReadDir DvbDir(DEV_DVB_BASE);
+  if (DvbDir.Ok()) {
+     struct dirent *a;
+     while ((a = DvbDir.Next()) != NULL) {
+           if (strstr(a->d_name, DEV_DVB_ADAPTER) == a->d_name) {
+              int Adapter = strtol(a->d_name + strlen(DEV_DVB_ADAPTER), NULL, 10);
+              cReadDir AdapterDir(AddDirectory(DEV_DVB_BASE, a->d_name));
+              if (AdapterDir.Ok()) {
+                 struct dirent *f;
+                 while ((f = AdapterDir.Next()) != NULL) {
+                       if (strstr(f->d_name, DEV_DVB_FRONTEND) == f->d_name) {
+                          int Frontend = strtol(f->d_name + strlen(DEV_DVB_FRONTEND), NULL, 10);
+                          Nodes.Append(strdup(cString::sprintf("%2d %2d", Adapter, Frontend)));
+                          }
+                       }
+                 }
+              }
+           }
+     }
   int Checked = 0;
   int Found = 0;
-  for (int Adapter = 0; ; Adapter++) {
-      for (int Frontend = 0; ; Frontend++) {
-          if (Exists(Adapter, Frontend)) {
-             if (Checked++ < MAXDVBDEVICES) {
-                if (UseDevice(NextCardIndex())) {
-                   if (Probe(Adapter, Frontend))
-                      Found++;
-                   }
-                else
-                   NextCardIndex(1); // skips this one
-                }
-             }
-          else if (Frontend == 0)
-             goto LastAdapter;
-          else
-             goto NextAdapter;
-          }
-      NextAdapter: ;
-      }
-LastAdapter:
+  if (Nodes.Size() > 0) {
+     Nodes.Sort();
+     for (int i = 0; i < Nodes.Size(); i++) {
+         int Adapter;
+         int Frontend;
+         if (2 == sscanf(Nodes[i], "%d %d", &Adapter, &Frontend)) {
+            if (Exists(Adapter, Frontend)) {
+               if (Checked++ < MAXDVBDEVICES) {
+                  if (UseDevice(NextCardIndex())) {
+                     if (Probe(Adapter, Frontend))
+                        Found++;
+                     }
+                  else
+                     NextCardIndex(1); // skips this one
+                  }
+               }
+            }
+         }
+     }
   NextCardIndex(MAXDVBDEVICES - Checked); // skips the rest
   if (Found > 0)
      isyslog("found %d DVB device%s", Found, Found > 1 ? "s" : "");
@@ -1130,7 +1153,7 @@ bool cDvbDevice::QueryDeliverySystems(int fd_frontend)
      LOG_ERROR;
      return false;
      }
-#if DVB_API_VERSION > 5 || DVB_API_VERSION_MINOR >= 5
+#if (DVB_API_VERSION << 8 | DVB_API_VERSION_MINOR) >= 0x0505
   dtv_property Frontend[1];
   memset(&Frontend, 0, sizeof(Frontend));
   dtv_properties CmdSeq;
@@ -1415,32 +1438,30 @@ bool cDvbDevice::ProvidesTransponder(const cChannel *Channel) const
 bool cDvbDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *NeedsDetachReceivers) const
 {
   bool result = false;
-  bool hasPriority = Priority < 0 || Priority > this->Priority();
+  bool hasPriority = Priority == IDLEPRIORITY || Priority > this->Priority();
   bool needsDetachReceivers = false;
   needsDetachBondedReceivers = false;
 
   if (dvbTuner && ProvidesTransponder(Channel)) {
      result = hasPriority;
-     if (Priority >= 0) {
-        if (Receiving(true)) {
+     if (Priority > IDLEPRIORITY) {
+        if (Receiving()) {
            if (dvbTuner->IsTunedTo(Channel)) {
-              if (Channel->Vpid() && !HasPid(Channel->Vpid()) || Channel->Apid(0) && !HasPid(Channel->Apid(0))) {
+              if (Channel->Vpid() && !HasPid(Channel->Vpid()) || Channel->Apid(0) && !HasPid(Channel->Apid(0)) || Channel->Dpid(0) && !HasPid(Channel->Dpid(0))) {
                  if (CamSlot() && Channel->Ca() >= CA_ENCRYPTED_MIN) {
                     if (CamSlot()->CanDecrypt(Channel))
                        result = true;
                     else
                        needsDetachReceivers = true;
                     }
-                 else if (!IsPrimaryDevice())
-                    result = true;
                  else
-                    result = Priority >= Setup.PrimaryLimit;
+                    result = true;
                  }
               else
-                 result = !IsPrimaryDevice() || Priority >= Setup.PrimaryLimit;
+                 result = true;
               }
            else
-              needsDetachReceivers = true;
+              needsDetachReceivers = Receiving();
            }
         if (result) {
            if (!BondingOk(Channel)) {
@@ -1452,7 +1473,7 @@ bool cDvbDevice::ProvidesChannel(const cChannel *Channel, int Priority, bool *Ne
                      }
                   }
               needsDetachBondedReceivers = true;
-              needsDetachReceivers = true;
+              needsDetachReceivers = Receiving();
               }
            }
         }
@@ -1569,21 +1590,43 @@ cDvbDeviceProbe::~cDvbDeviceProbe()
 
 uint32_t cDvbDeviceProbe::GetSubsystemId(int Adapter, int Frontend)
 {
-  cString FileName;
-  cReadLine ReadLine;
-  FILE *f = NULL;
   uint32_t SubsystemId = 0;
-  FileName = cString::sprintf("/sys/class/dvb/dvb%d.frontend%d/device/subsystem_vendor", Adapter, Frontend);
-  if ((f = fopen(FileName, "r")) != NULL) {
-     if (char *s = ReadLine.Read(f))
-        SubsystemId = strtoul(s, NULL, 0) << 16;
-     fclose(f);
-     }
-  FileName = cString::sprintf("/sys/class/dvb/dvb%d.frontend%d/device/subsystem_device", Adapter, Frontend);
-  if ((f = fopen(FileName, "r")) != NULL) {
-     if (char *s = ReadLine.Read(f))
-        SubsystemId |= strtoul(s, NULL, 0);
-     fclose(f);
+  cString FileName = cString::sprintf("/dev/dvb/adapter%d/frontend%d", Adapter, Frontend);
+  struct stat st;
+  if (stat(FileName, &st) == 0) {
+     cReadDir d("/sys/class/dvb");
+     if (d.Ok()) {
+        struct dirent *e;
+        while ((e = d.Next()) != NULL) {
+              if (strstr(e->d_name, "frontend")) {
+                 FileName = cString::sprintf("/sys/class/dvb/%s/dev", e->d_name);
+                 if (FILE *f = fopen(FileName, "r")) {
+                    cReadLine ReadLine;
+                    char *s = ReadLine.Read(f);
+                    fclose(f);
+                    unsigned Major;
+                    unsigned Minor;
+                    if (s && 2 == sscanf(s, "%u:%u", &Major, &Minor)) {
+                       if (((Major << 8) | Minor) == st.st_rdev) {
+                          FileName = cString::sprintf("/sys/class/dvb/%s/device/subsystem_vendor", e->d_name);
+                          if ((f = fopen(FileName, "r")) != NULL) {
+                             if (char *s = ReadLine.Read(f))
+                                SubsystemId = strtoul(s, NULL, 0) << 16;
+                             fclose(f);
+                             }
+                          FileName = cString::sprintf("/sys/class/dvb/%s/device/subsystem_device", e->d_name);
+                          if ((f = fopen(FileName, "r")) != NULL) {
+                             if (char *s = ReadLine.Read(f))
+                                SubsystemId |= strtoul(s, NULL, 0);
+                             fclose(f);
+                             }
+                          break;
+                          }
+                       }
+                    }
+                 }
+              }
+        }
      }
   return SubsystemId;
 }
