@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 2.21 2010/03/07 14:24:26 kls Exp $
+ * $Id: dvbplayer.c 2.27 2012/05/06 11:02:35 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -209,6 +209,7 @@ private:
   cUnbufferedFile *replayFile;
   double framesPerSecond;
   bool isPesRecording;
+  bool pauseLive;
   bool eof;
   bool firstPacket;
   ePlayModes playMode;
@@ -228,7 +229,7 @@ protected:
   virtual void Activate(bool On);
   virtual void Action(void);
 public:
-  cDvbPlayer(const char *FileName);
+  cDvbPlayer(const char *FileName, bool PauseLive);
   virtual ~cDvbPlayer();
   bool Active(void) { return cThread::Running(); }
   void Pause(void);
@@ -249,7 +250,7 @@ public:
 #define SPEED_MULT   12 // the speed multiplier
 int cDvbPlayer::Speeds[] = { 0, -2, -4, -8, 1, 2, 4, 12, 0 };
 
-cDvbPlayer::cDvbPlayer(const char *FileName)
+cDvbPlayer::cDvbPlayer(const char *FileName, bool PauseLive)
 :cThread("dvbplayer")
 {
   nonBlockingFileReader = NULL;
@@ -258,6 +259,7 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
   cRecording Recording(FileName);
   framesPerSecond = Recording.FramesPerSecond();
   isPesRecording = Recording.IsPesRecording();
+  pauseLive = PauseLive;
   eof = false;
   firstPacket = true;
   playMode = pmPlay;
@@ -275,13 +277,15 @@ cDvbPlayer::cDvbPlayer(const char *FileName)
      return;
   ringBuffer = new cRingBufferFrame(PLAYERBUFSIZE);
   // Create the index file:
-  index = new cIndexFile(FileName, false, isPesRecording);
+  index = new cIndexFile(FileName, false, isPesRecording, pauseLive);
   if (!index)
      esyslog("ERROR: can't allocate index");
   else if (!index->Ok()) {
      delete index;
      index = NULL;
      }
+  else if (PauseLive)
+     framesPerSecond = cRecording(FileName).FramesPerSecond(); // the fps rate might have changed from the default
 }
 
 cDvbPlayer::~cDvbPlayer()
@@ -400,14 +404,16 @@ void cDvbPlayer::Action(void)
   int LastReadIFrame = -1;
   int SwitchToPlayFrame = 0;
 
+  if (pauseLive)
+     Goto(0, true);
   while (Running()) {
         if (WaitingForData)
-           nonBlockingFileReader->WaitForDataMs(3); // this keeps the CPU load low, but reacts immediately on new data
+           WaitingForData = !nonBlockingFileReader->WaitForDataMs(3); // this keeps the CPU load low, but reacts immediately on new data
         else if (Sleep) {
            cPoller Poller;
            DevicePoll(Poller, 10);
            Sleep = false;
-           if (playMode == pmStill || playMode==pmPause)
+           if (playMode == pmStill || playMode == pmPause)
               cCondWait::SleepMs(3);
            }
         {
@@ -435,9 +441,9 @@ void cDvbPlayer::Action(void)
                          int NewIndex = readIndex + d;
                          if (NewIndex <= 0 && readIndex > 0)
                             NewIndex = 1; // make sure the very first frame is delivered
-                         NewIndex = index->GetNextIFrame(NewIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length, TimeShiftMode);
+                         NewIndex = index->GetNextIFrame(NewIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length);
                          if (NewIndex < 0 && TimeShiftMode && playDir == pdForward)
-                            SwitchToPlayFrame = Index;
+                            SwitchToPlayFrame = readIndex;
                          Index = NewIndex;
                          readIndependent = true;
                          }
@@ -446,7 +452,7 @@ void cDvbPlayer::Action(void)
                          if (!NextFile(FileNumber, FileOffset))
                             continue;
                          }
-                      else
+                      else if (!(TimeShiftMode && playDir == pdForward))
                          eof = true;
                       }
                    else if (index) {
@@ -480,16 +486,16 @@ void cDvbPlayer::Action(void)
                          }
                       readFrame = new cFrame(b, -r, ftUnknown, readIndex, Pts); // hands over b to the ringBuffer
                       }
-                   else if (r < 0 && errno == EAGAIN)
-                      WaitingForData = true;
-                   else {
-                      if (r == 0)
-                         eof = true;
-                      else if (r < 0 && FATALERRNO) {
+                   else if (r < 0) {
+                      if (errno == EAGAIN)
+                         WaitingForData = true;
+                      else if (FATALERRNO) {
                          LOG_ERROR;
                          break;
                          }
                       }
+                   else
+                      eof = true;
                    }
                 }
 
@@ -758,7 +764,7 @@ void cDvbPlayer::SkipSeconds(int Seconds)
      if (Index >= 0) {
         Index = max(Index + SecondsToFrames(Seconds, framesPerSecond), 0);
         if (Index > 0)
-           Index = index->GetNextIFrame(Index, false, NULL, NULL, NULL, true);
+           Index = index->GetNextIFrame(Index, false, NULL, NULL, NULL);
         if (Index >= 0)
            readIndex = Index - 1; // Action() will first increment it!
         }
@@ -821,8 +827,8 @@ bool cDvbPlayer::GetReplayMode(bool &Play, bool &Forward, int &Speed)
 
 // --- cDvbPlayerControl -----------------------------------------------------
 
-cDvbPlayerControl::cDvbPlayerControl(const char *FileName)
-:cControl(player = new cDvbPlayer(FileName))
+cDvbPlayerControl::cDvbPlayerControl(const char *FileName, bool PauseLive)
+:cControl(player = new cDvbPlayer(FileName, PauseLive))
 {
 }
 
