@@ -4,10 +4,10 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * Original author: Marco Schller <marco@lordzodiac.de>
+ * Original author: Marco Schluessler <marco@lordzodiac.de>
  * With some input from the "subtitle plugin" by Pekka Virtanen <pekka.virtanen@sci.fi>
  *
- * $Id: dvbsubtitle.c 2.21 2012/01/11 10:34:07 kls Exp $
+ * $Id: dvbsubtitle.c 2.32 2012/05/08 08:17:17 kls Exp $
  */
 
 
@@ -217,7 +217,7 @@ void cSubtitleObject::DecodeCharacterString(const uchar *Data, int NumberOfCodes
         char txt[NumberOfCodes + 1];
         char *p = txt;
         for (int i = 2; i < NumberOfCodes; ++i) {
-            char c = Data[i * 2 + 1] & 0xFF;
+            uchar c = Data[i * 2 + 1] & 0xFF;
             if (c == 0)
                break;
             if (' ' <= c && c <= '~' || c == '\n' || 0xA0 <= c)
@@ -491,13 +491,14 @@ void cSubtitleRegion::UpdateTextData(cSubtitleClut *Clut)
   const cPalette *palette = Clut ? Clut->GetPalette(Depth()) : NULL;
   for (cSubtitleObject *so = objects.First(); so && palette; so = objects.Next(so)) {
       if (Utf8StrLen(so->TextData()) > 0) {
-         const cFont *font = cFont::GetFont(fontOsd);
-         cBitmap *tmp = new cBitmap(font->Width(so->TextData()), font->Height(), Depth());
+         cFont *font = cFont::CreateFont(Setup.FontOsd, Setup.FontOsdSize);
+         cBitmap tmp(font->Width(so->TextData()), font->Height(), Depth());
          double factor = (double)lineHeight / font->Height();
-         tmp->DrawText(0, 0, so->TextData(), palette->Color(so->ForegroundPixelCode()), palette->Color(so->BackgroundPixelCode()), font);
-         tmp = tmp->Scaled(factor, factor, true);
-         DrawBitmap(so->X(), so->Y(), *tmp);
-         DELETENULL(tmp);
+         tmp.DrawText(0, 0, so->TextData(), palette->Color(so->ForegroundPixelCode()), palette->Color(so->BackgroundPixelCode()), font);
+         cBitmap *scaled = tmp.Scaled(factor, factor, true);
+         DrawBitmap(so->X(), so->Y(), *scaled);
+         delete scaled;
+         delete font;
          }
       }
 }
@@ -541,7 +542,6 @@ public:
   void SetPts(int64_t Pts) { pts = Pts; }
   void SetState(int State);
   void SetTimeout(int Timeout) { timeout = Timeout; }
-  void UpdateRegionPalette(cSubtitleClut *Clut);
   };
 
 cDvbSubtitlePage::cDvbSubtitlePage(int PageId)
@@ -620,7 +620,7 @@ void cDvbSubtitlePage::SetState(int State)
     case 0: // normal case - page update
          dbgpages("page update\n");
          break;
-    case 1: // aquisition point - page refresh
+    case 1: // acquisition point - page refresh
          dbgpages("page refresh\n");
          regions.Clear();
          break;
@@ -635,14 +635,6 @@ void cDvbSubtitlePage::SetState(int State)
     }
 }
 
-void cDvbSubtitlePage::UpdateRegionPalette(cSubtitleClut *Clut)
-{
-  for (cSubtitleRegion *sr = regions.First(); sr; sr = regions.Next(sr)) {
-      if (sr->ClutId() == Clut->ClutId())
-         sr->Replace(*Clut->GetPalette(sr->Bpp()));
-      }
-}
-
 // --- cDvbSubtitleAssembler -------------------------------------------------
 
 class cDvbSubtitleAssembler {
@@ -806,7 +798,6 @@ cDvbSubtitleConverter::cDvbSubtitleConverter(void)
   displayHeight = windowHeight = 576;
   windowHorizontalOffset = 0;
   windowVerticalOffset = 0;
-  SetOsdData();
   pages = new cList<cDvbSubtitlePage>;
   bitmaps = new cList<cDvbSubtitleBitmaps>;
   Start();
@@ -840,7 +831,6 @@ void cDvbSubtitleConverter::Reset(void)
   displayHeight = windowHeight = 576;
   windowHorizontalOffset = 0;
   windowVerticalOffset = 0;
-  SetOsdData();
   Unlock();
 }
 
@@ -980,9 +970,9 @@ tColor cDvbSubtitleConverter::yuv2rgb(int Y, int Cb, int Cr)
   Epb = (Cb - 128);
   Epr = (Cr - 128);
   /* ITU-R 709 */
-  Er = max(min(((298 * Ey             + 460 * Epr) / 256), 255), 0);
-  Eg = max(min(((298 * Ey -  55 * Epb - 137 * Epr) / 256), 255), 0);
-  Eb = max(min(((298 * Ey + 543 * Epb            ) / 256), 255), 0);
+  Er = constrain((298 * Ey             + 460 * Epr) / 256, 0, 255);
+  Eg = constrain((298 * Ey -  55 * Epb - 137 * Epr) / 256, 0, 255);
+  Eb = constrain((298 * Ey + 543 * Epb            ) / 256, 0, 255);
 
   return (Er << 16) | (Eg << 8) | Eb;
 }
@@ -995,7 +985,7 @@ void cDvbSubtitleConverter::SetOsdData(void)
   double VideoAspect;
   cDevice::PrimaryDevice()->GetOsdSize(OsdWidth, OsdHeight, OsdAspect);
   cDevice::PrimaryDevice()->GetVideoSize(VideoWidth, VideoHeight, VideoAspect);
-  if (OsdWidth == displayWidth && OsdHeight == displayHeight || DoubleEqual(VideoAspect, 1.0)) {
+  if (OsdWidth == displayWidth && OsdHeight == displayHeight) {
      osdFactorX = osdFactorY = 1.0;
      osdDeltaX = osdDeltaY = 0;
      }
@@ -1010,7 +1000,11 @@ void cDvbSubtitleConverter::SetOsdData(void)
 bool cDvbSubtitleConverter::AssertOsd(void)
 {
   LOCK_THREAD;
-  return osd || (osd = cOsdProvider::NewOsd(int(round(osdFactorX * windowHorizontalOffset + osdDeltaX)), int(round(osdFactorY * windowVerticalOffset + osdDeltaY)) + Setup.SubtitleOffset, OSD_LEVEL_SUBTITLES));
+  if (!osd) {
+     SetOsdData();
+     osd = cOsdProvider::NewOsd(int(round(osdFactorX * windowHorizontalOffset + osdDeltaX)), int(round(osdFactorY * windowVerticalOffset + osdDeltaY)) + Setup.SubtitleOffset, OSD_LEVEL_SUBTITLES);
+     }
+  return osd != NULL;
 }
 
 int cDvbSubtitleConverter::ExtractSegment(const uchar *Data, int Length, int64_t Pts)
@@ -1035,7 +1029,7 @@ int cDvbSubtitleConverter::ExtractSegment(const uchar *Data, int Length, int64_t
      if (!page) {
         page = new cDvbSubtitlePage(pageId);
         pages->Add(page);
-        dbgpages("Create SubtitlePage %d  (total pages = %d)\n", pageId, pages->Count());
+        dbgpages("Create SubtitlePage %d (total pages = %d)\n", pageId, pages->Count());
         }
      if (Pts)
         page->SetPts(Pts);
@@ -1049,7 +1043,6 @@ int cDvbSubtitleConverter::ExtractSegment(const uchar *Data, int Length, int64_t
             page->SetVersion(pageVersion);
             page->SetTimeout(pageTimeout);
             page->SetState(bs.GetBits(2));
-            page->regions.Clear();
             bs.SkipBits(2); // reserved
             dbgpages("Update page id %d version %d pts %"PRId64" timeout %d state %d\n", pageId, page->Version(), page->Pts(), page->Timeout(), page->State());
             while (!bs.IsEOF()) {
@@ -1113,7 +1106,7 @@ int cDvbSubtitleConverter::ExtractSegment(const uchar *Data, int Length, int64_t
             }
        case CLUT_DEFINITION_SEGMENT: {
             dbgsegments("CLUT_DEFINITION_SEGMENT\n");
-            cSubtitleClut *clut =  page->GetClutById(bs.GetBits(8), true);
+            cSubtitleClut *clut = page->GetClutById(bs.GetBits(8), true);
             int clutVersion = bs.GetBits(4);
             if (clutVersion == clut->Version())
                break; // no update
@@ -1156,7 +1149,6 @@ int cDvbSubtitleConverter::ExtractSegment(const uchar *Data, int Length, int64_t
                      clut->SetColor(8, clutEntryId, value);
                   }
             dbgcluts("\n");
-            page->UpdateRegionPalette(clut);
             break;
             }
        case OBJECT_DATA_SEGMENT: {
@@ -1289,9 +1281,16 @@ void cDvbSubtitleConverter::FinishPage(cDvbSubtitlePage *Page)
         else
            return; // unable to draw bitmaps
         }
-  if (Reduced) {
-     for (int i = 0; i < NumAreas; i++) {
-         cSubtitleRegion *sr = Page->regions.Get(i);
+  cDvbSubtitleBitmaps *Bitmaps = new cDvbSubtitleBitmaps(Page->Pts(), Page->Timeout(), Areas, NumAreas, osdFactorX, osdFactorY);
+  bitmaps->Add(Bitmaps);
+  for (int i = 0; i < NumAreas; i++) { 
+      cSubtitleRegion *sr = Page->regions.Get(i);
+      cSubtitleClut *clut = Page->GetClutById(sr->ClutId());
+      if (!clut)
+         continue;
+      sr->Replace(*clut->GetPalette(sr->Bpp()));
+      sr->UpdateTextData(clut);
+      if (Reduced) {
          if (sr->Bpp() != Areas[i].bpp) {
             if (sr->Level() <= Areas[i].bpp) {
                //TODO this is untested - didn't have any such subtitle stream
@@ -1307,11 +1306,6 @@ void cDvbSubtitleConverter::FinishPage(cDvbSubtitlePage *Page)
                }
             }
          }
-     }
-  cDvbSubtitleBitmaps *Bitmaps = new cDvbSubtitleBitmaps(Page->Pts(), Page->Timeout(), Areas, NumAreas, osdFactorX, osdFactorY);
-  bitmaps->Add(Bitmaps);
-  for (cSubtitleRegion *sr = Page->regions.First(); sr; sr = Page->regions.Next(sr)) {
-      sr->UpdateTextData(Page->GetClutById(sr->ClutId()));
       int posX = sr->HorizontalAddress();
       int posY = sr->VerticalAddress();
       if (sr->Width() > 0 && sr->Height() > 0) {
