@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: si.h 2.3 2010/02/13 10:31:34 kls Exp $
+ *   $Id: si.h 2.5 2012/01/11 11:35:17 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -134,6 +134,19 @@ enum DescriptorTag {
                DTSDescriptorTag = 0x7B,
                AACDescriptorTag = 0x7C,
                ExtensionDescriptorTag = 0x7F,
+ // Extension descriptors
+               ImageIconDescriptorTag = 0x00,
+               CpcmDeliverySignallingDescriptor = 0x01,
+               CPDescriptorTag = 0x02,
+               CPIdentifierDescriptorTag = 0x03,
+               T2DeliverySystemDescriptorTag = 0x04,
+               SHDeliverySystemDescriptorTag = 0x05,
+               SupplementaryAudioDescriptorTag = 0x06,
+               NetworkChangeNotifyDescriptorTag = 0x07,
+               MessageDescriptorTag = 0x08,
+               TargetRegionDescriptorTag = 0x09,
+               TargetRegionNameDescriptorTag = 0x0A,
+               ServiceRelocatedDescriptorTag = 0x0B,
 
  // Defined by ETSI TS 102 812 (MHP)
                // They once again start with 0x00 (see page 234, MHP specification)
@@ -153,6 +166,8 @@ enum DescriptorTag {
                MHP_PrefetchDescriptorTag = 0x0C,
                MHP_DelegatedApplicationDescriptorTag = 0x0E,
                MHP_ApplicationStorageDescriptorTag = 0x10,
+               MHP_SimpleApplicationLocationDescriptorTag = 0x15,
+               MHP_SimpleApplicationBoundaryDescriptorTag = 0x17,
   // Premiere private Descriptor Tags
                PremiereContentTransmissionDescriptorTag = 0xF2,
 
