@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.c 2.23 2011/08/15 09:27:39 kls Exp $
+ * $Id: osd.c 2.31 2012/06/02 13:32:38 kls Exp $
  */
 
 #include "osd.h"
@@ -40,6 +40,16 @@ tColor HsvToColor(double H, double S, double V)
      }
 }
 
+tColor RgbShade(tColor Color, double Factor)
+{
+  double f = fabs(constrain(Factor, -1.0, 1.0));
+  double w = Factor > 0 ? f * 0xFF : 0;
+  return (Color & 0xFF000000) |
+         (min(0xFF, int((1 - f) * ((Color >> 16) & 0xFF) + w + 0.5)) << 16) |
+         (min(0xFF, int((1 - f) * ((Color >>  8) & 0xFF) + w + 0.5)) <<  8) |
+         (min(0xFF, int((1 - f) * ( Color        & 0xFF) + w + 0.5))      );
+}
+
 #define USE_ALPHA_LUT
 #ifdef USE_ALPHA_LUT
 // Alpha blending with lookup table (by Reinhard Nissl <rnissl@gmx.de>)
@@ -254,6 +264,7 @@ cBitmap::cBitmap(int Width, int Height, int Bpp, int X0, int Y0)
   bitmap = NULL;
   x0 = X0;
   y0 = Y0;
+  width = height = 0;
   SetSize(Width, Height);
 }
 
@@ -262,6 +273,7 @@ cBitmap::cBitmap(const char *FileName)
   bitmap = NULL;
   x0 = 0;
   y0 = 0;
+  width = height = 0;
   LoadXpm(FileName);
 }
 
@@ -270,6 +282,7 @@ cBitmap::cBitmap(const char *const Xpm[])
   bitmap = NULL;
   x0 = 0;
   y0 = 0;
+  width = height = 0;
   SetXpm(Xpm);
 }
 
@@ -550,11 +563,15 @@ void cBitmap::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Color
      if (Width || Height) {
         limit = x + cw - x0;
         if (Width) {
-           if ((Alignment & taLeft) != 0)
-              ;
+           if ((Alignment & taLeft) != 0) {
+              if ((Alignment & taBorder) != 0)
+                 x += max(h / TEXT_ALIGN_BORDER, 1);
+              }
            else if ((Alignment & taRight) != 0) {
               if (w < Width)
                  x += Width - w;
+              if ((Alignment & taBorder) != 0)
+                 x -= max(h / TEXT_ALIGN_BORDER, 1);
               }
            else { // taCentered
               if (w < Width)
@@ -812,11 +829,11 @@ cBitmap *cBitmap::Scaled(double FactorX, double FactorY, bool AntiAlias)
   // Fixed point scaling code based on www.inversereality.org/files/bitmapscaling.pdf
   // by deltener@mindtremors.com
   cBitmap *b = new cBitmap(int(round(Width() * FactorX)), int(round(Height() * FactorY)), Bpp(), X0(), Y0());
-  b->Replace(*this); // copy palette
   int RatioX = (Width() << 16) / b->Width();
   int RatioY = (Height() << 16) / b->Height();
   if (!AntiAlias || FactorX <= 1.0 && FactorY <= 1.0) {
      // Downscaling - no anti-aliasing:
+     b->Replace(*this); // copy palette
      tIndex *DestRow = b->bitmap;
      int SourceY = 0;
      for (int y = 0; y < b->Height(); y++) {
@@ -834,6 +851,7 @@ cBitmap *cBitmap::Scaled(double FactorX, double FactorY, bool AntiAlias)
   else {
      // Upscaling - anti-aliasing:
      b->SetBpp(8);
+     b->Replace(*this); // copy palette (must be done *after* SetBpp()!)
      int SourceY = 0;
      for (int y = 0; y < b->Height() - 1; y++) {
          int SourceX = 0;
@@ -1007,7 +1025,7 @@ void cPixmap::SetLayer(int Layer)
 void cPixmap::SetAlpha(int Alpha)
 {
   Lock();
-  Alpha = min(max(Alpha, ALPHA_TRANSPARENT), ALPHA_OPAQUE);
+  Alpha = constrain(Alpha, ALPHA_TRANSPARENT, ALPHA_OPAQUE);
   if (Alpha != alpha) {
      MarkViewPortDirty(viewPort);
      alpha = Alpha;
@@ -1276,11 +1294,15 @@ void cPixmapMemory::DrawText(const cPoint &Point, const char *s, tColor ColorFg,
   if (Width || Height) {
      limit = x + cw;
      if (Width) {
-        if ((Alignment & taLeft) != 0)
-           ;
+        if ((Alignment & taLeft) != 0) {
+           if ((Alignment & taBorder) != 0)
+              x += max(h / TEXT_ALIGN_BORDER, 1);
+           }
         else if ((Alignment & taRight) != 0) {
            if (w < Width)
               x += Width - w;
+           if ((Alignment & taBorder) != 0)
+              x -= max(h / TEXT_ALIGN_BORDER, 1);
            }
         else { // taCentered
            if (w < Width)
@@ -1378,10 +1400,10 @@ void cPixmapMemory::DrawEllipse(const cRect &Rect, tColor Color, int Quadrants)
           case  0:
           case  6: DrawRectangle(cRect(cx - x, cy - y, 2 * x + 1,       1), Color); if (Quadrants == 6) break;
           case  8: DrawRectangle(cRect(cx - x, cy + y, 2 * x + 1,       1), Color); break;
-          case -1: DrawRectangle(cRect(cx + x, cy - y, x2 - x + 1,      1), Color); break;
+          case -1: DrawRectangle(cRect(cx + x, cy - y, rx - x + 1,      1), Color); break;
           case -2: DrawRectangle(cRect(x1,     cy - y, cx - x - x1 + 1, 1), Color); break;
           case -3: DrawRectangle(cRect(x1,     cy + y, cx - x - x1 + 1, 1), Color); break;
-          case -4: DrawRectangle(cRect(cx + x, cy + y, x2 - x + 1,      1), Color); break;
+          case -4: DrawRectangle(cRect(cx + x, cy + y, rx - x + 1,      1), Color); break;
           default: ;
           }
         y++;
@@ -1413,10 +1435,10 @@ void cPixmapMemory::DrawEllipse(const cRect &Rect, tColor Color, int Quadrants)
           case  0:
           case  6: DrawRectangle(cRect(cx - x, cy - y, 2 * x + 1,       1), Color); if (Quadrants == 6) break;
           case  8: DrawRectangle(cRect(cx - x, cy + y, 2 * x + 1,       1), Color); break;
-          case -1: DrawRectangle(cRect(cx + x, cy - y, x2 - x + 1,      1), Color); break;
+          case -1: DrawRectangle(cRect(cx + x, cy - y, rx - x + 1,      1), Color); break;
           case -2: DrawRectangle(cRect(x1,     cy - y, cx - x - x1 + 1, 1), Color); break;
           case -3: DrawRectangle(cRect(x1,     cy + y, cx - x - x1 + 1, 1), Color); break;
-          case -4: DrawRectangle(cRect(cx + x, cy + y, x2 - x + 1,      1), Color); break;
+          case -4: DrawRectangle(cRect(cx + x, cy + y, rx - x + 1,      1), Color); break;
           default: ;
           }
         x++;
@@ -1647,8 +1669,8 @@ void cOsd::SetOsdPosition(int Left, int Top, int Width, int Height)
 {
   osdLeft = Left;
   osdTop = Top;
-  osdWidth = min(max(Width, MINOSDWIDTH), MAXOSDWIDTH);
-  osdHeight = min(max(Height, MINOSDHEIGHT), MAXOSDHEIGHT);
+  osdWidth = constrain(Width, MINOSDWIDTH, MAXOSDWIDTH);
+  osdHeight = constrain(Height, MINOSDHEIGHT, MAXOSDHEIGHT);
 }
 
 void cOsd::SetAntiAliasGranularity(uint FixedColors, uint BlendColors)
@@ -1700,11 +1722,13 @@ void cOsd::DestroyPixmap(cPixmap *Pixmap)
 
 cPixmap *cOsd::AddPixmap(cPixmap *Pixmap)
 {
-  LOCK_PIXMAPS;
-  if (numPixmaps < MAXOSDPIXMAPS)
-     return pixmaps[numPixmaps++] = Pixmap;
-  else
-     esyslog("ERROR: too many OSD pixmaps requested (maximum is %d)", MAXOSDPIXMAPS);
+  if (Pixmap) {
+     LOCK_PIXMAPS;
+     if (numPixmaps < MAXOSDPIXMAPS)
+        return pixmaps[numPixmaps++] = Pixmap;
+     else
+        esyslog("ERROR: too many OSD pixmaps requested (maximum is %d)", MAXOSDPIXMAPS);
+     }
   return NULL;
 }
 
@@ -1982,7 +2006,7 @@ void cOsdProvider::UpdateOsdSize(bool Force)
      Setup.FontSmlSize = int(round(Height * Setup.FontSmlSizeP));
      cFont::SetFont(fontOsd, Setup.FontOsd, Setup.FontOsdSize);
      cFont::SetFont(fontFix, Setup.FontFix, Setup.FontFixSize);
-     cFont::SetFont(fontSml, Setup.FontSml, Setup.FontSmlSize);
+     cFont::SetFont(fontSml, Setup.FontSml, min(Setup.FontSmlSize, Setup.FontOsdSize));
      oldWidth = Width;
      oldHeight = Height;
      oldAspect = Aspect;
