@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.h 2.15 2011/12/04 13:38:17 kls Exp $
+ * $Id: osd.h 2.17 2012/06/02 10:32:38 kls Exp $
  */
 
 #ifndef __OSD_H
@@ -25,6 +25,7 @@
 #define ALPHA_TRANSPARENT  0x00
 #define ALPHA_OPAQUE       0xFF
 #define IS_OPAQUE(c)       ((c >> 24) == ALPHA_OPAQUE)
+#define TEXT_ALIGN_BORDER  10 // fraction of the font height used for sizing border
 
 enum {
                    //AARRGGBB
@@ -69,6 +70,14 @@ inline tColor RgbToColor(double R, double G, double B)
   return RgbToColor(uint8_t(0xFF * R), uint8_t(0xFF * G), uint8_t(0xFF * B));
 }
 
+tColor RgbShade(tColor Color, double Factor);
+   ///< Returns a brighter (Factor > 0) or darker (Factor < 0) version
+   ///< of the given Color.
+   ///< If Factor is 0.0, the return value is the unchanged Color,
+   ///< If Factor is 1.0, white is returned.
+   ///< If Factor is -1.0, black is returned.
+   ///< The alpha value of Color is returned unchanged.
+
 tColor HsvToColor(double H, double S, double V);
    ///< Converts the given Hue (0..360), Saturation (0..1) and Value (0..1)
    ///< to an RGB tColor value. The alpha value of the result is 0x00, so
@@ -151,6 +160,7 @@ enum eTextAlignment { taCenter  = 0x00,
                       taRight   = 0x02,
                       taTop     = 0x04,
                       taBottom  = 0x08,
+                      taBorder  = 0x10, // keeps some distance from the left or right alignment edge
                       taDefault = taTop | taLeft
                     };
 
