@@ -4,13 +4,14 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.c 2.2 2010/01/17 11:36:12 kls Exp $
+ * $Id: osdbase.c 2.6 2012/04/23 09:41:22 kls Exp $
  */
 
 #include "osdbase.h"
 #include <string.h>
 #include "device.h"
 #include "i18n.h"
+#include "menuitems.h"
 #include "remote.h"
 #include "status.h"
 
@@ -22,9 +23,6 @@ cOsdItem::cOsdItem(eOSState State)
   state = State;
   selectable = true;
   fresh = true;
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-  subMenu = 0;
-#endif /* SETUP & PINPLUGIN */
 }
 
 cOsdItem::cOsdItem(const char *Text, eOSState State, bool Selectable)
@@ -34,23 +32,8 @@ cOsdItem::cOsdItem(const char *Text, eOSState State, bool Selectable)
   selectable = Selectable;
   fresh = true;
   SetText(Text);
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-  subMenu = 0;
-#endif /* SETUP & PINPLUGIN */
 }
 
-#if defined (USE_SETUP) && defined (USE_PINPLUGIN)
-cOsdItem::cOsdItem(const char *Text, eOSState State, cSubMenuNode* SubMenu)
-{
-  text = NULL;
-  state = State;
-  selectable = true;
-  fresh = true;
-  SetText(Text);
-  subMenu = SubMenu;
-}
-#endif /* SETUP & PINPLUGIN */
-
 cOsdItem::~cOsdItem()
 {
   free(text);
@@ -89,23 +72,22 @@ void cOsdObject::Show(void)
 
 cSkinDisplayMenu *cOsdMenu::displayMenu = NULL;
 int cOsdMenu::displayMenuCount = 0;
-int cOsdMenu::displayMenuItems = 0;//XXX dynamic???
 
 cOsdMenu::cOsdMenu(const char *Title, int c0, int c1, int c2, int c3, int c4)
 {
   isMenu = true;
   digit = 0;
-#ifdef USE_LIEMIEXT
-  key_nr = -1;
-#endif /* LIEMIEXT */
   hasHotkeys = false;
+  displayMenuItems = 0;
   title = NULL;
+  menuCategory = mcUnknown;
   SetTitle(Title);
   SetCols(c0, c1, c2, c3, c4);
   first = 0;
   current = marked = -1;
   subMenu = NULL;
   helpRed = helpGreen = helpYellow = helpBlue = NULL;
+  helpDisplayed = false;
   status = NULL;
   if (!displayMenuCount++)
      SetDisplayMenu();
@@ -118,13 +100,15 @@ cOsdMenu::~cOsdMenu()
   free(status);
   displayMenu->Clear();
   cStatus::MsgOsdClear();
-#ifdef USE_GRAPHTFT
-  cStatus::MsgOsdMenuDestroy();
-#endif /* GRAPHTFT */
   if (!--displayMenuCount)
      DELETENULL(displayMenu);
 }
 
+void cOsdMenu::SetMenuCategory(eMenuCategory MenuCategory)
+{
+  menuCategory = MenuCategory;
+}
+
 void cOsdMenu::SetDisplayMenu(void)
 {
   if (displayMenu) {
@@ -132,7 +116,6 @@ void cOsdMenu::SetDisplayMenu(void)
      delete displayMenu;
      }
   displayMenu = Skins.Current()->DisplayMenu();
-  displayMenuItems = displayMenu->MaxItems();
 }
 
 const char *cOsdMenu::hk(const char *s)
@@ -143,11 +126,7 @@ const char *cOsdMenu::hk(const char *s)
         digit = -1; // prevents automatic hotkeys - input already has them
      if (digit >= 0) {
         digit++;
-#ifdef USE_LIEMIEXT
-        buffer = cString::sprintf(" %2d%s %s", digit, (digit > 9) ? "" : " ", s);
-#else
         buffer = cString::sprintf(" %c %s", (digit < 10) ? '0' + digit : ' ' , s);
-#endif /* LIEMIEXT */
         s = buffer;
         }
      }
@@ -182,6 +161,15 @@ void cOsdMenu::SetTitle(const char *Title)
   title = strdup(Title);
 }
 
+void cOsdMenu::DisplayHelp(bool Force)
+{
+  if (!helpDisplayed || Force) {
+     displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
+     cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+     helpDisplayed = true;
+     }
+}
+
 void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, const char *Blue)
 {
   // strings are NOT copied - must be constants!!!
@@ -189,8 +177,7 @@ void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, c
   helpGreen  = Green;
   helpYellow = Yellow;
   helpBlue   = Blue;
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  DisplayHelp(true);
 }
 
 void cOsdMenu::Del(int Index)
@@ -230,14 +217,13 @@ void cOsdMenu::Display(void)
   displayMenu->SetMessage(mtStatus, NULL);
   displayMenu->Clear();
   cStatus::MsgOsdClear();
-#ifdef USE_GRAPHTFT
-  cStatus::MsgOsdMenuDisplay(MenuKind());
-#endif /* GRAPHTFT */
+  if (menuCategory != displayMenu->MenuCategory())
+     displayMenu->SetMenuCategory(menuCategory);
+  displayMenuItems = displayMenu->MaxItems();
   displayMenu->SetTabs(cols[0], cols[1], cols[2], cols[3], cols[4]);//XXX
   displayMenu->SetTitle(title);
   cStatus::MsgOsdTitle(title);
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  DisplayHelp(true);
   int count = Count();
   if (count > 0) {
      int ni = 0;
@@ -294,6 +280,12 @@ void cOsdMenu::DisplayCurrent(bool Current)
         cStatus::MsgOsdCurrentItem(item->Text());
      if (!Current)
         item->SetFresh(true); // leaving the current item resets 'fresh'
+     if (cMenuEditItem *MenuEditItem = dynamic_cast<cMenuEditItem *>(item)) {
+        if (!MenuEditItem->DisplayHelp())
+           DisplayHelp();
+        else
+           helpDisplayed = false;
+        }
      }
 }
 
@@ -480,79 +472,20 @@ void cOsdMenu::Mark(void)
      }
 }
 
-#ifdef USE_LIEMIEXT
-#define MENUKEY_TIMEOUT 1500
-#endif /* LIEMIEXT */
-
 eOSState cOsdMenu::HotKey(eKeys Key)
 {
-#ifdef USE_LIEMIEXT
-  bool match = false;
-  bool highlight = false;
-  int  item_nr;
-  int  i;
-
-  if (Key == kNone) {
-     if (lastActivity.TimedOut())
-        Key = kOk;
-     else
-        return osContinue;
-     }
-  else {
-     lastActivity.Set(MENUKEY_TIMEOUT);
-     }
-  for (cOsdItem *item = Last(); item; item = Prev(item)) {
-#else
   for (cOsdItem *item = First(); item; item = Next(item)) {
-#endif /* LIEMIEXT */
       const char *s = item->Text();
-#ifdef USE_LIEMIEXT
-      i = 0;
-      item_nr = 0;
-      if (s && (s = skipspace(s)) != '\0' && '0' <= s[i] && s[i] <= '9') {
-         do {
-            item_nr = item_nr * 10 + (s[i] - '0');
-            }
-         while ( !((s[++i] == '\t')||(s[i] == ' ')) && (s[i] != '\0') && ('0' <= s[i]) && (s[i] <= '9'));
-         if ((Key == kOk) && (item_nr == key_nr)) {
-#else
       if (s && (s = skipspace(s)) != NULL) {
          if (*s == Key - k1 + '1') {
-#endif /* LIEMIEXT */
             current = item->Index();
             RefreshCurrent();
             Display();
             cRemote::Put(kOk, true);
-#ifdef USE_LIEMIEXT
-            key_nr = -1;
-#endif /* LIEMIEXT */
             break;
             }
-#ifdef USE_LIEMIEXT
-         else if (Key != kOk) {
-            if (!highlight && (item_nr == (Key - k0))) {
-               highlight = true;
-               current = item->Index();
-               }
-            if (!match && (key_nr == -1) && ((item_nr / 10) == (Key - k0))) {
-               match = true;
-               key_nr = (Key - k0);
-               }
-            else if (((key_nr == -1) && (item_nr == (Key - k0))) || (!match && (key_nr >= 0) && (item_nr == (10 * key_nr + Key - k0)))) {
-               current = item->Index();
-               cRemote::Put(kOk, true);
-               key_nr = -1;
-               break;
-               }
-            }
-#endif /* LIEMIEXT */
          }
       }
-#ifdef USE_LIEMIEXT
-  if ((!match) && (Key != kNone)) {
-     key_nr = -1;
-     }
-#endif /* LIEMIEXT */
   return osContinue;
 }
 
@@ -590,14 +523,9 @@ eOSState cOsdMenu::ProcessKey(eKeys Key)
         return state;
         }
      }
-  switch (Key) {
-#ifdef USE_LIEMIEXT
-    case kNone:
-    case k0...k9: return hasHotkeys ? HotKey(Key) : osUnknown;
-#else
+  switch (int(Key)) {
     case k0:      return osUnknown;
     case k1...k9: return hasHotkeys ? HotKey(Key) : osUnknown;
-#endif /* LIEMIEXT */
     case kUp|k_Repeat:
     case kUp:   CursorUp();   break;
     case kDown|k_Repeat:
