@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 2.43 2012/01/15 11:04:24 kls Exp $
+ * $Id: recording.c 2.56 2012/06/03 09:51:27 kls Exp $
  */
 
 #include "recording.h"
@@ -153,7 +153,7 @@ void AssertFreeDiskSpace(int Priority, bool Force)
            cRecording *r = DeletedRecordings.First();
            cRecording *r0 = NULL;
            while (r) {
-                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only remove recordings that will actually increase the free video disk space
+                 if (r->IsOnVideoDirectoryFileSystem()) { // only remove recordings that will actually increase the free video disk space
                     if (!r0 || r->Start() < r0->Start())
                        r0 = r;
                     }
@@ -180,7 +180,7 @@ void AssertFreeDiskSpace(int Priority, bool Force)
            cRecording *r = Recordings.First();
            cRecording *r0 = NULL;
            while (r) {
-                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only delete recordings that will actually increase the free video disk space
+                 if (r->IsOnVideoDirectoryFileSystem()) { // only delete recordings that will actually increase the free video disk space
                     if (!r->IsEdited() && r->Lifetime() < MAXLIFETIME) { // edited recordings and recordings with MAXLIFETIME live forever
                        if ((r->Lifetime() == 0 && Priority > r->Priority()) || // the recording has no guaranteed lifetime and the new recording has higher priority
                            (r->Lifetime() > 0 && (time(NULL) - r->Start()) / SECSINDAY >= r->Lifetime())) { // the recording's guaranteed lifetime has expired
@@ -617,6 +617,7 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
   channel = Timer->Channel()->Number();
   instanceId = InstanceId;
   isPesRecording = false;
+  isOnVideoDirectoryFileSystem = -1; // unknown
   framesPerSecond = DEFAULTFRAMESPERSECOND;
   numFrames = -1;
   deleted = 0;
@@ -677,6 +678,7 @@ cRecording::cRecording(const char *FileName)
   priority = MAXPRIORITY; // assume maximum in case there is no info file
   lifetime = MAXLIFETIME;
   isPesRecording = false;
+  isOnVideoDirectoryFileSystem = -1; // unknown
   framesPerSecond = DEFAULTFRAMESPERSECOND;
   numFrames = -1;
   deleted = 0;
@@ -723,7 +725,9 @@ cRecording::cRecording(const char *FileName)
            }
         fclose(f);
         }
-     else if (errno != ENOENT)
+     else if (errno == ENOENT)
+        info->ownEvent->SetTitle(name);
+     else
         LOG_ERROR_STR(*InfoFileName);
 #ifdef SUMMARYFALLBACK
      // fall back to the old 'summary.vdr' if there was no 'info.vdr':
@@ -875,11 +879,11 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level) cons
         s = name;
      cString Length("");
      if (NewIndicator) {
-        int Seconds = max(0, LengthInSeconds());
+        int Minutes = max(0, (LengthInSeconds() + 30) / 60);
         Length = cString::sprintf("%c%d:%02d",
                    Delimiter,
-                   Seconds / 3600,
-                   Seconds / 60 % 60
+                   Minutes / 60,
+                   Minutes % 60
                    );
         }
      titleBuffer = strdup(cString::sprintf("%02d.%02d.%02d%c%02d:%02d%s%c%c%s",
@@ -950,6 +954,13 @@ bool cRecording::IsEdited(void) const
   return *s == '%';
 }
 
+bool cRecording::IsOnVideoDirectoryFileSystem(void) const
+{
+  if (isOnVideoDirectoryFileSystem < 0)
+     isOnVideoDirectoryFileSystem = ::IsOnVideoDirectoryFileSystem(FileName());
+  return isOnVideoDirectoryFileSystem;
+}
+
 void cRecording::ReadInfo(void)
 {
   info->Read();
@@ -1059,10 +1070,21 @@ int cRecording::LengthInSeconds(void) const
 {
   int nf = NumFrames();
   if (nf >= 0)
-     return int((nf / FramesPerSecond() + 30) / 60) * 60;
+     return int(nf / FramesPerSecond());
   return -1;
 }
 
+int cRecording::FileSizeMB(void) const
+{
+  if (fileSizeMB < 0) {
+     int fs = DirSizeMB(FileName());
+     if (time(NULL) - LastModifiedTime(FileName()) < MININDEXAGE)
+        return fs; // check again later for ongoing recordings
+     fileSizeMB = fs;
+     }
+  return fileSizeMB;
+}
+
 // --- cRecordings -----------------------------------------------------------
 
 cRecordings Recordings;
@@ -1109,43 +1131,37 @@ void cRecordings::ScanVideoDir(const char *DirName, bool Foreground, int LinkLev
   cReadDir d(DirName);
   struct dirent *e;
   while ((Foreground || Running()) && (e = d.Next()) != NULL) {
-        if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-           cString buffer = AddDirectory(DirName, e->d_name);
-           struct stat st;
-           if (lstat(buffer, &st) == 0) {
-              int Link = 0;
-              if (S_ISLNK(st.st_mode)) {
-                 if (LinkLevel > MAX_LINK_LEVEL) {
-                    isyslog("max link level exceeded - not scanning %s", *buffer);
-                    continue;
-                    }
-                 Link = 1;
-                 buffer = ReadLink(buffer);
-                 if (!*buffer)
-                    continue;
-                 if (stat(buffer, &st) != 0)
-                    continue;
+        cString buffer = AddDirectory(DirName, e->d_name);
+        struct stat st;
+        if (lstat(buffer, &st) == 0) {
+           int Link = 0;
+           if (S_ISLNK(st.st_mode)) {
+              if (LinkLevel > MAX_LINK_LEVEL) {
+                 isyslog("max link level exceeded - not scanning %s", *buffer);
+                 continue;
                  }
-              if (S_ISDIR(st.st_mode)) {
-                 if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
-                    cRecording *r = new cRecording(buffer);
-                    if (r->Name()) {
-                       r->NumFrames(); // initializes the numFrames member
-                       Lock();
-                       Add(r);
-                       ChangeState();
-                       Unlock();
-                       if (deleted) {
-                          r->fileSizeMB = DirSizeMB(buffer);
-                          r->deleted = time(NULL);
-                          }
-                       }
-                    else
-                       delete r;
+              Link = 1;
+              if (stat(buffer, &st) != 0)
+                 continue;
+              }
+           if (S_ISDIR(st.st_mode)) {
+              if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
+                 cRecording *r = new cRecording(buffer);
+                 if (r->Name()) {
+                    r->NumFrames(); // initializes the numFrames member
+                    r->FileSizeMB(); // initializes the fileSizeMB member
+                    if (deleted)
+                       r->deleted = time(NULL);
+                    Lock();
+                    Add(r);
+                    ChangeState();
+                    Unlock();
                     }
                  else
-                    ScanVideoDir(buffer, Foreground, LinkLevel + Link);
+                    delete r;
                  }
+              else
+                 ScanVideoDir(buffer, Foreground, LinkLevel + Link);
               }
            }
         }
@@ -1220,12 +1236,13 @@ void cRecordings::DelByName(const char *FileName)
      char *ext = strrchr(recording->fileName, '.');
      if (ext) {
         strncpy(ext, DELEXT, strlen(ext));
-        recording->fileSizeMB = DirSizeMB(recording->FileName());
-        recording->deleted = time(NULL);
-        DeletedRecordings.Add(recording);
+        if (access(recording->FileName(), F_OK) == 0) {
+           recording->deleted = time(NULL);
+           DeletedRecordings.Add(recording);
+           recording = NULL; // to prevent it from being deleted below
+           }
         }
-     else
-        delete recording;
+     delete recording;
      ChangeState();
      TouchUpdate();
      }
@@ -1244,12 +1261,33 @@ int cRecordings::TotalFileSizeMB(void)
   int size = 0;
   LOCK_THREAD;
   for (cRecording *recording = First(); recording; recording = Next(recording)) {
-      if (recording->fileSizeMB > 0 && IsOnVideoDirectoryFileSystem(recording->FileName()))
-         size += recording->fileSizeMB;
+      int FileSizeMB = recording->FileSizeMB();
+      if (FileSizeMB > 0 && recording->IsOnVideoDirectoryFileSystem())
+         size += FileSizeMB;
       }
   return size;
 }
 
+double cRecordings::MBperMinute(void)
+{
+  int size = 0;
+  int length = 0;
+  LOCK_THREAD;
+  for (cRecording *recording = First(); recording; recording = Next(recording)) {
+      if (recording->IsOnVideoDirectoryFileSystem()) {
+         int FileSizeMB = recording->FileSizeMB();
+         if (FileSizeMB > 0) {
+            int LengthInSeconds = recording->LengthInSeconds();
+            if (LengthInSeconds > 0) {
+               size += FileSizeMB;
+               length += LengthInSeconds;
+               }
+            }
+         }
+      }
+  return (size && length) ? double(size) * 60 / length : -1;
+}
+
 void cRecordings::ResetResume(const char *ResumeFileName)
 {
   LOCK_THREAD;
@@ -1395,13 +1433,17 @@ cMark *cMarks::GetNext(int Position)
 
 const char *cRecordingUserCommand::command = NULL;
 
-void cRecordingUserCommand::InvokeCommand(const char *State, const char *RecordingFileName)
+void cRecordingUserCommand::InvokeCommand(const char *State, const char *RecordingFileName, const char *SourceFileName)
 {
   if (command) {
-     cString cmd = cString::sprintf("%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"));
-     isyslog("executing '%s'", *cmd);
-     SystemExec(cmd);
-     }
+    cString cmd;
+    if (SourceFileName)
+       cmd = cString::sprintf("%s %s \"%s\" \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"), *strescape(SourceFileName, "\\\"$"));
+    else
+       cmd = cString::sprintf("%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"));
+    isyslog("executing '%s'", *cmd);
+    SystemExec(cmd);
+  }
 }
 
 // --- cIndexFileGenerator ---------------------------------------------------
@@ -1529,9 +1571,6 @@ void cIndexFileGenerator::Action(void)
 
 #define INDEXFILESUFFIX     "/index"
 
-// The number of frames to stay off the end in case of time shift:
-#define INDEXSAFETYLIMIT 150 // frames
-
 // The maximum time to wait before giving up while catching up on an index file:
 #define MAXINDEXCATCHUP   8 // seconds
 
@@ -1558,8 +1597,9 @@ struct tIndexTs {
 
 #define MAXWAITFORINDEXFILE     10 // max. time to wait for the regenerated index file (seconds)
 #define INDEXFILECHECKINTERVAL 500 // ms between checks for existence of the regenerated index file
+#define INDEXFILETESTINTERVAL   10 // ms between tests for the size of the index file in case of pausing live video
 
-cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording)
+cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording, bool PauseLive)
 :resumeFile(FileName, IsPesRecording)
 {
   f = -1;
@@ -1570,6 +1610,12 @@ cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording)
   indexFileGenerator = NULL;
   if (FileName) {
      fileName = IndexFileName(FileName, isPesRecording);
+     if (!Record && PauseLive) {
+        // Wait until the index file contains at least two frames:
+        time_t tmax = time(NULL) + MAXWAITFORINDEXFILE;
+        while (time(NULL) < tmax && FileSize(fileName) < off_t(2 * sizeof(tIndexTs)))
+              cCondWait::SleepMs(INDEXFILETESTINTERVAL);
+        }
      int delta = 0;
      if (!Record && access(fileName, R_OK) != 0) {
         // Index file doesn't exist, so try to regenerate it:
@@ -1724,7 +1770,7 @@ bool cIndexFile::CatchUp(int Index)
             }
          else
             LOG_ERROR_STR(*fileName);
-         if (Index < last - (i ? 2 * INDEXSAFETYLIMIT : 0) || Index > 10 * INDEXSAFETYLIMIT) // keep off the end in case of "Pause live video"
+         if (Index < last)
             break;
          cCondWait::SleepMs(1000);
          }
@@ -1771,13 +1817,13 @@ bool cIndexFile::Get(int Index, uint16_t *FileNumber, off_t *FileOffset, bool *I
   return false;
 }
 
-int cIndexFile::GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber, off_t *FileOffset, int *Length, bool StayOffEnd)
+int cIndexFile::GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber, off_t *FileOffset, int *Length)
 {
   if (CatchUp()) {
      int d = Forward ? 1 : -1;
      for (;;) {
          Index += d;
-         if (Index >= 0 && Index < last - ((Forward && StayOffEnd) ? INDEXSAFETYLIMIT : 0)) {
+         if (Index >= 0 && Index < last) {
             if (index[Index].independent) {
                uint16_t fn;
                if (!FileNumber)
