@@ -198,11 +198,7 @@ void cSectionHandler::Action(void)
                   if (fh) {
                      // Read section data:
                      unsigned char buf[4096]; // max. allowed size for any EIT section
-#ifdef USE_MCLI
-                     int r = device->ReadFilter(fh->handle, buf, sizeof(buf));
-#else
                      int r = safe_read(fh->handle, buf, sizeof(buf));
-#endif /* MCLI */
                      if (!DeviceHasLock)
                         continue; // we do the read anyway, to flush any data that might have come from a different transponder
                      if (r > 3) { // minimum number of bytes necessary to get section length
