@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: sources.h 1.4 2005/05/14 09:30:41 kls Exp $
+ * $Id: sources.h 2.3 2010/03/07 13:53:11 kls Exp $
  */
 
 #ifndef __SOURCES_H
@@ -15,26 +15,20 @@
 class cSource : public cListObject {
 public:
   enum eSourceType {
-    stNone  = 0x0000,
-#ifdef USE_IPTV
-    stPlug  = 0x2000,
-#endif /* IPTV */
-    stCable = 0x4000,
-    stSat   = 0x8000,
-    stTerr  = 0xC000,
-#ifdef USE_IPTV
-    st_Mask = 0xE000,
-#else
-    st_Mask = 0xC000,
-#endif /* IPTV */
-    st_Neg  = 0x0800,
-    st_Pos  = 0x07FF,
+    stNone  = 0x00000000,
+    stAtsc  = ('A' << 24),
+    stCable = ('C' << 24),
+    stSat   = ('S' << 24),
+    stTerr  = ('T' << 24),
+    st_Mask = 0xFF000000,
+    st_Pos  = 0x0000FFFF,
     };
 private:
   int code;
   char *description;
 public:
   cSource(void);
+  cSource(char Source, const char *Description);
   ~cSource();
   int Code(void) const { return code; }
   const char *Description(void) const { return description; }
@@ -42,12 +36,11 @@ public:
   static cString ToString(int Code);
   static int FromString(const char *s);
   static int FromData(eSourceType SourceType, int Position = 0, bool East = false);
-#ifdef USE_IPTV
-  static bool IsPlug(int Code) { return (Code & st_Mask) == stPlug; }
-#endif /* IPTV */
+  static bool IsAtsc(int Code) { return (Code & st_Mask) == stAtsc; }
   static bool IsCable(int Code) { return (Code & st_Mask) == stCable; }
   static bool IsSat(int Code) { return (Code & st_Mask) == stSat; }
   static bool IsTerr(int Code) { return (Code & st_Mask) == stTerr; }
+  static bool IsType(int Code, char Source) { return int(Code & st_Mask) == (int(Source) << 24); }
   };
 
 class cSources : public cConfig<cSource> {
