@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 2.13 2012/01/12 15:02:46 kls Exp $
+ * $Id: svdrp.c 2.19 2012/05/12 11:55:18 kls Exp $
  */
 
 #include "svdrp.h"
@@ -256,8 +256,6 @@ const char *HelpPages[] = {
   "    used to easily activate or deactivate a timer.",
   "MOVC <number> <to>\n"
   "    Move a channel to a new position.",
-  "MOVT <number> <to>\n"
-  "    Move a timer to a new position.",
   "NEWC <settings>\n"
   "    Create a new channel. Settings must be in the same format as returned\n"
   "    by the LSTC command.",
@@ -434,7 +432,7 @@ void cSVDRP::Reply(int Code, const char *fmt, ...)
      if (Code != 0) {
         va_list ap;
         va_start(ap, fmt);
-        cString buffer = cString::sprintf(fmt, ap);
+        cString buffer = cString::vsprintf(fmt, ap);
         va_end(ap);
         const char *s = buffer;
         while (s && *s) {
@@ -603,9 +601,13 @@ void cSVDRP::CmdCLRE(const char *Option)
         Reply(501, "Undefined channel \"%s\"", Option);
      }
   else {
-     cSchedules::ClearAll();
      cEitFilter::SetDisableUntil(time(NULL) + EITDISABLETIME);
-     Reply(250, "EPG data cleared");
+     if (cSchedules::ClearAll()) {
+        Reply(250, "EPG data cleared");
+        cEitFilter::SetDisableUntil(time(NULL) + EITDISABLETIME);
+        }
+     else
+        Reply(451, "Error while clearing EPG data");
      }
 }
 
@@ -660,22 +662,26 @@ void cSVDRP::CmdDELR(const char *Option)
 {
   if (*Option) {
      if (isnumber(Option)) {
-        cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+        cRecording *recording = recordings.Get(strtol(Option, NULL, 10) - 1);
         if (recording) {
            cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
            if (!rc) {
-              if (recording->Delete()) {
-                 Reply(250, "Recording \"%s\" deleted", Option);
-                 ::Recordings.DelByName(recording->FileName());
+              if (!cCutter::Active(recording->FileName())) {
+                 if (recording->Delete()) {
+                    Reply(250, "Recording \"%s\" deleted", Option);
+                    Recordings.DelByName(recording->FileName());
+                    }
+                 else
+                    Reply(554, "Error while deleting recording!");
                  }
               else
-                 Reply(554, "Error while deleting recording!");
+                 Reply(550, "Recording \"%s\" is being edited", Option);
               }
            else
               Reply(550, "Recording \"%s\" is in use by timer %d", Option, rc->Timer()->Index() + 1);
            }
         else
-           Reply(550, "Recording \"%s\" not found%s", Option, Recordings.Count() ? "" : " (use LSTR before deleting)");
+           Reply(550, "Recording \"%s\" not found%s", Option, recordings.Count() ? "" : " (use LSTR before deleting)");
         }
      else
         Reply(501, "Error in recording number \"%s\"", Option);
@@ -717,7 +723,7 @@ void cSVDRP::CmdEDIT(const char *Option)
 {
   if (*Option) {
      if (isnumber(Option)) {
-        cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+        cRecording *recording = recordings.Get(strtol(Option, NULL, 10) - 1);
         if (recording) {
            cMarks Marks;
            if (Marks.Load(recording->FileName(), recording->FramesPerSecond(), recording->IsPesRecording()) && Marks.Count()) {
@@ -734,7 +740,7 @@ void cSVDRP::CmdEDIT(const char *Option)
               Reply(554, "No editing marks defined");
            }
         else
-           Reply(550, "Recording \"%s\" not found%s", Option, Recordings.Count() ? "" : " (use LSTR before editing)");
+           Reply(550, "Recording \"%s\" not found%s", Option, recordings.Count() ? "" : " (use LSTR before editing)");
         }
      else
         Reply(501, "Error in recording number \"%s\"", Option);
@@ -1070,10 +1076,10 @@ void cSVDRP::CmdLSTE(const char *Option)
 
 void cSVDRP::CmdLSTR(const char *Option)
 {
-  bool recordings = Recordings.Update(true);
+  recordings.Update(true);
   if (*Option) {
      if (isnumber(Option)) {
-        cRecording *recording = Recordings.Get(strtol(Option, NULL, 10) - 1);
+        cRecording *recording = recordings.Get(strtol(Option, NULL, 10) - 1);
         if (recording) {
            FILE *f = fdopen(file, "w");
            if (f) {
@@ -1091,11 +1097,11 @@ void cSVDRP::CmdLSTR(const char *Option)
      else
         Reply(501, "Error in recording number \"%s\"", Option);
      }
-  else if (recordings) {
-     cRecording *recording = Recordings.First();
+  else if (recordings.Count()) {
+     cRecording *recording = recordings.First();
      while (recording) {
-           Reply(recording == Recordings.Last() ? 250 : -250, "%d %s", recording->Index() + 1, recording->Title(' ', true));
-           recording = Recordings.Next(recording);
+           Reply(recording == recordings.Last() ? 250 : -250, "%d %s", recording->Index() + 1, recording->Title(' ', true));
+           recording = recordings.Next(recording);
            }
      }
   else
@@ -1283,12 +1289,6 @@ void cSVDRP::CmdMOVC(const char *Option)
      Reply(501, "Missing channel number");
 }
 
-void cSVDRP::CmdMOVT(const char *Option)
-{
-  //TODO combine this with menu action
-  Reply(502, "MOVT not yet implemented");
-}
-
 void cSVDRP::CmdNEWC(const char *Option)
 {
   if (*Option) {
@@ -1367,11 +1367,11 @@ void cSVDRP::CmdPLAY(const char *Option)
      char c = *option;
      *option = 0;
      if (isnumber(num)) {
-        cRecording *recording = Recordings.Get(strtol(num, NULL, 10) - 1);
+        cRecording *recording = recordings.Get(strtol(num, NULL, 10) - 1);
         if (recording) {
            if (c)
               option = skipspace(++option);
-           cReplayControl::SetRecording(NULL, NULL);
+           cReplayControl::SetRecording(NULL);
            cControl::Shutdown();
            if (*option) {
               int pos = 0;
@@ -1383,13 +1383,13 @@ void cSVDRP::CmdPLAY(const char *Option)
               else
                  resume.Save(pos);
               }
-           cReplayControl::SetRecording(recording->FileName(), recording->Title());
+           cReplayControl::SetRecording(recording->FileName());
            cControl::Launch(new cReplayControl);
            cControl::Attach();
            Reply(250, "Playing recording \"%s\" [%s]", num, recording->Title());
            }
         else
-           Reply(550, "Recording \"%s\" not found%s", num, Recordings.Count() ? "" : " (use LSTR before playing)");
+           Reply(550, "Recording \"%s\" not found%s", num, recordings.Count() ? "" : " (use LSTR before playing)");
         }
      else
         Reply(501, "Error in recording number \"%s\"", num);
@@ -1604,6 +1604,7 @@ void cSVDRP::Execute(char *Cmd)
         Reply(PUTEhandler->Status(), "%s", PUTEhandler->Message());
         DELETENULL(PUTEhandler);
         }
+     cEitFilter::SetDisableUntil(time(NULL) + EITDISABLETIME); // re-trigger the timeout, in case there is very much EPG data
      return;
      }
   // skip leading whitespace:
@@ -1632,7 +1633,6 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("MODC"))  CmdMODC(s);
   else if (CMD("MODT"))  CmdMODT(s);
   else if (CMD("MOVC"))  CmdMOVC(s);
-  else if (CMD("MOVT"))  CmdMOVT(s);
   else if (CMD("NEWC"))  CmdNEWC(s);
   else if (CMD("NEWT"))  CmdNEWT(s);
   else if (CMD("NEXT"))  CmdNEXT(s);
