@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 2.5 2011/08/06 13:13:54 kls Exp $
+ * $Id: timers.c 2.9 2012/04/25 09:02:03 kls Exp $
  */
 
 #include "timers.h"
@@ -142,7 +142,7 @@ cTimer& cTimer::operator= (const cTimer &Timer)
 
 int cTimer::Compare(const cListObject &ListObject) const
 {
-  cTimer *ti = (cTimer *)&ListObject;
+  const cTimer *ti = (const cTimer *)&ListObject;
   time_t t1 = StartTime();
   time_t t2 = ti->StartTime();
   int r = t1 - t2;
@@ -384,13 +384,14 @@ time_t cTimer::SetTime(time_t t, int SecondsFromMidnight)
   return mktime(&tm);
 }
 
-char *cTimer::SetFile(const char *File)
+void cTimer::SetFile(const char *File)
 {
   if (!isempty(File))
      Utf8Strn0Cpy(file, File, sizeof(file));
-  return file;
 }
 
+#define EITPRESENTFOLLOWINGRATE 10 // max. seconds between two occurrences of the "EIT present/following table for the actual multiplex" (2s by the standard, using some more for safety)
+
 bool cTimer::Matches(time_t t, bool Directly, int Margin) const
 {
   startTime = stopTime = 0;
@@ -434,8 +435,12 @@ bool cTimer::Matches(time_t t, bool Directly, int Margin) const
         if (Margin || !Directly) {
            startTime = event->StartTime();
            stopTime = event->EndTime();
-           if (!Margin)
-              return event->IsRunning(true);
+           if (!Margin) { // this is an actual check
+              if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGRATE)) // VPS control can only work with up-to-date events...
+                 return event->IsRunning(true);
+              else
+                 return startTime <= t && t < stopTime; // ...otherwise we fall back to normal timer handling
+              }
            }
         }
      return startTime <= t + Margin && t < stopTime; // must stop *before* stopTime to allow adjacent timers
@@ -591,11 +596,42 @@ void cTimer::SetInVpsMargin(bool InVpsMargin)
   inVpsMargin = InVpsMargin;
 }
 
+void cTimer::SetDay(time_t Day)
+{
+  day = Day;
+}
+
+void cTimer::SetWeekDays(int WeekDays)
+{
+  weekdays = WeekDays;
+}
+
+void cTimer::SetStart(int Start)
+{
+  start = Start;
+}
+
+void cTimer::SetStop(int Stop)
+{
+  stop = Stop;
+}
+
 void cTimer::SetPriority(int Priority)
 {
   priority = Priority;
 }
 
+void cTimer::SetLifetime(int Lifetime)
+{
+  lifetime = Lifetime;
+}
+
+void cTimer::SetAux(const char *Aux)
+{
+  free(aux);
+  aux = strdup(Aux);
+}
+
 void cTimer::SetDeferred(int Seconds)
 {
   deferred = time(NULL) + Seconds;
@@ -784,3 +820,18 @@ void cTimers::DeleteExpired(void)
         }
   lastDeleteExpired = time(NULL);
 }
+
+// --- cSortedTimers ---------------------------------------------------------
+
+static int CompareTimers(const void *a, const void *b)
+{
+  return (*(const cTimer **)a)->Compare(**(const cTimer **)b);
+}
+
+cSortedTimers::cSortedTimers(void)
+:cVector(Timers.Count())
+{
+  for (const cTimer *Timer = Timers.First(); Timer; Timer = Timers.Next(Timer))
+      Append(Timer);
+  Sort(CompareTimers);
+}
