@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.h 2.1 2011/08/06 12:59:32 kls Exp $
+ * $Id: timers.h 2.4 2012/04/15 13:21:31 kls Exp $
  */
 
 #ifndef __TIMERS_H
@@ -75,7 +75,7 @@ public:
   bool DayMatches(time_t t) const;
   static time_t IncDay(time_t t, int Days);
   static time_t SetTime(time_t t, int SecondsFromMidnight);
-  char *SetFile(const char *File);
+  void SetFile(const char *File);
   bool Matches(time_t t = 0, bool Directly = false, int Margin = 0) const;
   int Matches(const cEvent *Event, int *Overlap = NULL) const;
   bool Expired(void) const;
@@ -86,7 +86,13 @@ public:
   void SetRecording(bool Recording);
   void SetPending(bool Pending);
   void SetInVpsMargin(bool InVpsMargin);
+  void SetDay(time_t Day);
+  void SetWeekDays(int WeekDays);
+  void SetStart(int Start);
+  void SetStop(int Stop);
   void SetPriority(int Priority);
+  void SetLifetime(int Lifetime);
+  void SetAux(const char *Aux);
   void SetDeferred(int Seconds);
   void SetFlags(uint Flags);
   void ClrFlags(uint Flags);
@@ -129,4 +135,9 @@ public:
 
 extern cTimers Timers;
 
+class cSortedTimers : public cVector<const cTimer *> {
+public:
+  cSortedTimers(void);
+  };
+
 #endif //__TIMERS_H
